/*	MIRS9805
**	Task01  : Ultra Sonic Sensor Control Task
**	Version 0.06 (99/07/20) 
**	by Fred on 18 June 1999
**
**	history : 
**	Ver 0.07   99/07/20   Beta version
**	Ver 0.06   99/07/12   Bug fixed
**	Ver 0.05   99/07/10   S̓IɏC
**	Ver 0.04   99/06/26   que @\̉
**	Ver 0.03   99/06/25   Corections
**	Ver 0.02   99/06/22   Comments added
**	Ver 0.01   99/06/18   First version
*/


#include "mirx68k.h" 		/* MIRX68K header file                 */ 
#include "hard.h"		/* Hardware function library header file*/
#include "defs.h"		/* MIRS9805's defination header file	*/

#define QUE_MAX		4       /* Que's maximum input			*/
#define USS_MAX		4       /* USS's quantity			*/
#define USS_TIMEOUT	999     /* USS's timeout			*/
#define HARDWARE_ERR	888     /* Hardware error occured		*/
#define TOO_NEAR	777     /* distance is too near			*/

static int que[QUE_MAX];        /* que's array				*/
static int uss_count[USS_MAX];  /* USS count's array			*/
static int uss_no=0;            /* the # USS in use 			*/
static int busy=0;              /* USS : 1 (busy measuring) @ 0 (idle)	*/
static int que_now=0;		/* next que #				*/


void inzt01()			/* initialize initialization task	*/
{
	int i;
	uss_init();
	sys0(1,0x1101);			/* Task initialization registration */
	sys5(1,0,USS_INT_VCT,0,0);	/* Interupt task initialization     */
	sys8(1,0,0);			/* #1's result(length in cm)  	    */		sys8(1,1,0);			/* Number of times measured by #1   */
	sys8(1,2,0);			/* #2's result(length in cm)        */
	sys8(1,3,0);			/* Number of times measured by #2   */
	sys8(1,4,0);			/* #3's result(length in cm)        */
	sys8(1,5,0);			/* Number of times measured by #3   */ 
	sys8(1,6,0);			/* #4's result(length in cm)        */
	sys8(1,7,0);			/* Number of times measured by #4   */
	sys8(1,8,0);			/* Quantity of numbers in que	    */
	sys8(1,9,0);			/* Bit coded que 		    */
	for (i=0; i<QUE_MAX; i++)	/* Que intialization  		    */
		que[i] = -1;
	for (i=0; i<USS_MAX; i++)	/* uss_count intialization	    */			uss_count[i] = 0;
	sys11(1);
}

void nrmt01(int cm, int scm)			/* initialize normal task   */	{
	int m,n;
	int flag=0;			
	if(cm==1){				/* start measuring process  */
		if(scm>=USS_MAX || scm<0){	/* if scm's value is invalid*/
			return;			/* exit this task	    */
		}
		if(que_now<QUE_MAX){			/*if que is not full*/
			que[que_now]=scm;	/*register in que #m*/
			que_now++;
			flag=1;			/* make USS measure         */
		}
	}else if(cm==2){			/* stop measuring process   */
		if(busy==1){			/* if USS is measuring      */
			uss_stop();		/* stop measuring           */
		}
		busy=0;				/* make USS idle	    */
		for (m=0; m<QUE_MAX; m++){	/* clear que 	            */
			que[m] = -1;	
		}
		que_now=0;
	}else if(cm==3){			/* called from timer task   */
		if(que_now>0)
			flag=1;			/* make USS measure	    */
	}
	
	if(flag==1){
		if(busy==1){			/* USS is busy measuring    */ 
		}else{
			uss_no=que[0];		/* USS is idle		    */
			if(uss_no>=0){
				uss_start(uss_no);/* start measuring	    */
				busy=1;			/* set busy flag	    */
				que_now--;
				for (n=0; n<que_now; n++){/* que's content shifted  */
					que[n] = que[n+1];
				}
				que[que_now]=-1;
			}
		}
	}
}

void timt01(int cm, int scm)			/* timer task NOP	    */
{
}

interrupt void intt01()				/* interruption sensed      */
{
	int n,b;
	int distance;
	uss_stop();				/* stop USS                 */
	if(uss_chk_int() == 1){			/*perfect interruption sensed*/
		distance=uss_data();
		if(uss_chk_conect() == 0){	/* US wave received   	    */ 
			if(distance > 22){ 	/* distance is more than 22cm*/
				sys8(1,uss_no<<1,distance); /*distance in BB*/
			}
			else{
				sys8(1,uss_no<<1,TOO_NEAR);/*distance too near*/
			}
		}else{
			sys8(1,uss_no<<1,USS_TIMEOUT);	   /*'time_out' in BB*/
		}
	}else{	
		sys8(1,uss_no<<1,HARDWARE_ERR);	/*write 'hardware_err' in BB*/
	}
	uss_count[uss_no]++ ;			/* count 		    */ 
	sys8(1,(uss_no<<1)+1,uss_count[uss_no]);	/*'uss_count' in BB */
	if(que[0]!=-1){				/* if que is not empty	    */
		sys1(1,PY_USS,3,0);		/* call normal task         */
	}
	b=0;
	for(n=0;n<4;n++){			
		b=b*10;				/* que ̒g܂Ƃ߂	    */
		if(que[n]!=-1){
			b+=que[n];
		} else {
			b+=9;			/* Ȃ 9  */
		}
	}
	sys8(1,8,que_now);				/* 	*/
	sys8(1,9,b);				/* que	*/
	busy=0;
}


