/*	MIRS9805
**	Task02	:	ȈʒuoTask
**	Version 0.10  (99/08/17)
**	by Fred on 25 June 1999
**
**	history :
**	Ver 0.10	99/08/17	ƏC
**	Ver 0.09	99/07/20	Beta version (B_CCW,B_CW )
**	Ver 0.08	99/07/17	ꕔ萔 defs.h Ɉړ
**	Ver 0.07	99/07/15	x_sh,y_shx_thre,y_threɂ
**	Ver 0.06	99/07/13	x,y limitx_sh,y_shɂ
**	Ver 0.05	99/07/12	Bug fixed
**	Ver 0.04	99/07/02	Commands added,INIT_TH changed
**	Ver 0.03	99/06/29	Corrections
**	Ver 0.02	99/06/27	Distance from wall measurement with USS
**	Ver 0.01	99/06/25	First version 
*/


#include "mirx68k.h"  		/* MIRX68K header file			*/
#include "hard.h"		/* Hardware function library header file*/
#include "defs.h"		/* MIRS9805 defination header file	*/

/*#define B_CCW	0.128f      	*//* ^CԊûP^Q [m] for CCW	*/
/*#define B_CW	0.12423f 	*//* ^CԊûP^Q [m] for CW	*/

#define B_CW	0.094301f
#define B_CCW	0.097101f


#define Kv	COEF			/* velocity̌W		*/
#define Kw_CW	(COEF/B_CW)		/* omega(turn right̎)̌W	*/ 
#define Kw_CCW	(COEF/B_CCW)		/* omega(turn left̎)̌W	*/

#define DEG(x)	(x/PI*180.0f)	/* 180/(change from radian to degree) */

#define INIT_X	(0.0f)		/* Initial X-coordinate			*/
#define INIT_Y	(-115.0f)	/* Initial Y-coordinate			*/
#define INIT_TH	(PI)		/* Initial Theta coordinate		*/

extern float cosf(float);	/* cosinus function			*/
extern float sinf(float);	/* sinus function			*/

static int 	flag=0;		/* Coordinate correction flag		*/

static float	x_thre=120.0f;	/* x-axis limit				*/
static float	y_thre=120.0f;	/* y-axis limit 			*/

static float	re_x=INIT_X;	/* X-coordinate[cm]			*/
static float	re_y=INIT_Y;	/* Y-coordinate[cm]			*/
static float	re_th=INIT_TH;	/* angle [rad]				*/
static float	w;		/* angular velocity(omega)[rad/sec]	*/
static float	old;		/*  */
static int 	ussf=0;		/* USS flag				 */
static float 	len;

void inzt02()			/* initialize initialization task	*/
{
	sys0(2,0x1110);			/* Task initialization registration */
	sys2(2,TIMER_RE,0,0);		/* Interrupt task registration	*/
	sys4(2);			/* Count Stop for safety	*/
	sys3(2,1);			/* Count Start			*/
	sys8(2,0,(int)(INIT_X));	/* X-coordinate data		*/
	sys8(2,1,(int)(INIT_Y));	/* Y-coordinate data		*/
	sys8(2,2,(int)(DEG(INIT_TH)));	/* Theta(angular) data 		*/
	sys8(2,3,0);			/* Right Rotaring Encoder data	*/
	sys8(2,4,0);			/* Left Rotaring Encoder data	*/
	sys8(2,5,0);			/* Touch Sensor(TS) data	*/
	sys8(2,6,0);			/* MIRS's direction		*/
	sys8(2,7,0);			/* velocity			*/
	sys8(2,8,0);			/* angular velocity		*/
	sys8(2,9,0);			/* not used			*/
	w=0;				/* initialize omega		*/
	len=0;
	re_ts_data();			/* 			*/
}


void nrmt02(int cm, int scm)		/* normal task initialization	*/
{
	if(cm==1){
		re_x =*((float *)scm);	/* read value of x		*/
		re_y =*((float *)scm+1);/* read value of y		*/
		re_th=*((float *)scm+2);/* read value of theta		*/
		sys8(2,0,(int)(re_x));	/* write x-coordinate in BB     */
		sys8(2,1,(int)(re_y));	/* write y-coordinate in BB     */
		sys8(2,2,(int)(DEG(re_th)));  /*write theta(angle) in BB	*/
	}else if(cm==2){
		x_thre =(*((float *)scm));
		y_thre =(*((float *)scm+1));
	}
}


void timt02(int cm, int scm)			/* initialize timer task*/
{
	RE_TS_DATA	data;			/* RE and TS data	*/
	float	v;				/* speed[m/s]		*/
	float  	w_old;				/* omega(old)		*/
	int ts_in;				/* TS input data	*/
	int self_dir;				/* MIRS facing direction*/
	static int n=0;				/* measure with USS #n 	*/

	data=re_ts_data();			/* call RE and TS data	*/
	ts_in=0;
	BIT_W(ts_in,0,BIT_R(data.ts,0));	/* ̓f[^H	*/
	BIT_W(ts_in,1,BIT_R(data.ts,1));
	BIT_W(ts_in,2,BIT_R(data.ts,2));
	sys8(2,5,ts_in);			/* write TS data	*/

	v = Kv*(float)(data.r+data.l);		/* calculation of v	*/
	w_old = w;				/* save omega		*/
	w = ((data.r>data.l)*Kw_CCW+(data.r<data.l)*Kw_CW)*(float)(data.r-data.l);
						/* calculation of omega	*/
	re_th += ((w+w_old)*DT)/2.f;		/*renewal of angle theta*/

	/* re_th+=(w*DT); */
	while(re_th<0.f){			/* make 0<theta<Q 	*/
		re_th += PI2;
	}
	while(re_th>=PI2){			/* make 0<theta<Q	*/
		re_th -= PI2;
	}
	
	re_x += (v*DT*sinf(re_th))*100.f;	/*calculate x-coordinate*/
	re_y -= (v*DT*cosf(re_th))*100.f;	/*calculate y-coordinate*/
	
	if(re_x >  x_thre) re_x =  x_thre;	/* limit X-axis		*/
	if(re_x < -x_thre) re_x = -x_thre;
	if(re_y >  y_thre) re_y =  y_thre;	/* limit Y-axis		*/
	if(re_y < -y_thre) re_y = -y_thre;
	
	len+=(v*DT);
		
	self_dir = (int)(((int)(DEG(re_th))+45)%360)/90;
					/* calculate self direction	*/

	sys8(2,0,(int)(re_x));		/* write x-coordinate in BB     */
	sys8(2,1,(int)(re_y));		/* write y-coordinate in BB     */
	sys8(2,2,(int)(DEG(re_th)));	/* write theta(angle) in BB	*/
	sys8(2,3,data.r);		/* write right RE in BB		*/
	sys8(2,4,data.l);		/* write left RE in BB  	*/
	sys8(2,5,ts_in);		/* write Touch Sensor(TS) data	*/
	sys8(2,6,self_dir);		/* write self direction in BB	*/
	sys8(2,7,(int)(v*100));		/* write velocity to BB		*/
	sys8(2,8,(int)(DEG(w)));		/* write angular velocity in BB */
	sys8(2,9,((int)len*100));	/* write len in BB	*/
}


interrupt void intt02()			/* interrupt task NOP		*/
{
}

/* End Of File ^o^ */
